/**
 * @file
 * Signature of file given keyrings of sender and recipients.
 */

(function ($) {
    'use strict';
    Drupal.behaviors.proc = {
        attach: function (context, settings) {
            let procJsLabels      = Drupal.settings.proc.proc_labels,
                passDrupal        = Drupal.settings.proc.proc_pass,
                privkey           = Drupal.settings.proc.proc_privkey,
                keyRingId         = Drupal.settings.proc.proc_keyring_id;

            if (!(window.FileReader)) {
                alert(procJsLabels.proc_fileapi_err_msg);
            }

            // Reset messages and action.
            $('input#edit-pass').once().on(
                'focusin', function () {
                    $('.messages').remove();
                    this.value = '';
                }
            );


            if (document.getElementById('edit-button')) {
                document.getElementById('edit-button').disabled = "TRUE";
            }

            function handleFileSelect(evt) {
                let procJsLabels        = Drupal.settings.proc.proc_labels,
                    files               = evt.target.files,
                    postMaxSizeBytes    = Drupal.settings.proc.proc_post_max_size_bytes,
                    fileEntityMaxSize   = parseInt(Drupal.settings.proc.proc_file_entity_max_filesize, 10),
                    fileSize            = parseInt(files[0].size, 10),
                    postMaxSizeBytesInt = parseInt(postMaxSizeBytes, 10),
                    // Assuming ciphertexts are at least 4 times bigger than
                    // their plaintexts:
                    dynamicMaximumSize  = postMaxSizeBytesInt / 4;

                document.getElementById('edit-button').value = procJsLabels.proc_button_state_processing;
                $('label[for=edit-pc-upload-description]')[0].innerText =
                `${procJsLabels.proc_size} ${files[0].size} ${procJsLabels.proc_max_encryption_size_unit}
                ${procJsLabels.proc_type} ${files[0].type}
                ${procJsLabels.proc_last_modified} ${files[0].lastModifiedDate}`;

                let realMaxSize = dynamicMaximumSize;
                if (fileSize > dynamicMaximumSize || fileSize > fileEntityMaxSize) {
                    if (fileEntityMaxSize < dynamicMaximumSize){
                        realMaxSize = fileEntityMaxSize;
                    }
                    $("form#-proc-sign-file").prepend('<div class="messages error">' + `${procJsLabels.proc_max_encryption_size} ${realMaxSize} ${procJsLabels.proc_max_encryption_size_unit}` + '</div>');
                    document.getElementById('edit-button').value = procJsLabels.proc_save_button_label;
                    return;
                }

                let myFile        = files[0],
                    reader        = new FileReader(),
                    fileByteArray = [];

                reader.readAsArrayBuffer(myFile);
                reader.onloadend = async function (evt) {
                    if (evt.target.readyState == FileReader.DONE) {

                        let passphrase       = passDrupal.concat($('input[name=pass]')[0].value);
                        const privKeyObj = (await openpgp.key.readArmored(privkey)).keys[0];

                        await privKeyObj.decrypt(passphrase).catch(
                            function (err) {
                                $('form#-proc-sign-file').prepend(`<div class="messages error">${Drupal.t(err)}</div>`);
                                throw new Error('Could not decrypt private key.');
                            }
                        );

                        if (!$('#proc-decrypting-info')[0]) {
                            $('form#-proc-sign-file').prepend(`<div class="messages info proc-info" id="proc-decrypting-info">${procJsLabels.proc_introducing_signature}</div>`);
                        }

                        const recipientsPubkeys = [];

                        let arrayBuffer               = evt.target.result,
                            array                     = new Uint8Array(arrayBuffer),
                            // At this moment we only know about validated recipient UIDs
                            // and the time stamps of their keys:
                            recipientsUidsKeysChanged = JSON.parse(Drupal.settings.proc.proc_recipients_pubkeys_changed),
                            remoteKey                 = [],
                            userIdIterator;

                        for (let i = 0; i < array.length; i++) {
                            fileByteArray.push(array[i]);
                        }
                        // False for production.
                        openpgp.config.debug        = false;
                        openpgp.config.show_comment = false;
                        openpgp.config.show_version = false;

                        for (userIdIterator in recipientsUidsKeysChanged) {
                            let localKey = localStorage.getItem(`proc.key_user_id.${userIdIterator}.${recipientsUidsKeysChanged[userIdIterator]}`);
                            if (localKey){
                                recipientsPubkeys.push(localKey);
                            }
                            else{
                                let storageKeys = Object.keys(localStorage);
                                if (storageKeys.length > 0) {
                                    storageKeys.forEach(
                                        function (storageKey,storageKeyIndex){
                                            if (storageKey.startsWith(`proc.key_user_id.${userIdIterator}`)){
                                                localStorage.removeItem(storageKeys[storageKeyIndex]);
                                            }
                                        }
                                    );
                                }
                                remoteKey.push(userIdIterator);
                            }
                        }

                        let pubkeysJson;

                        if (remoteKey.length > 0){
                            let remoteKeyCsv = remoteKey.join(",");

                            const pubKeyAjax = async (remoteKeyCsv) => {
                                let response = await fetch(
                                    `${window.location.origin + Drupal.settings.basePath}proc/api/getpubkey/${remoteKeyCsv}/uid`
                                );
                                pubkeysJson = await response.json();

                                if (pubkeysJson.pubkey.length > 0){
                                    pubkeysJson.pubkey.forEach(
                                        function (pubkey,index){
                                            recipientsPubkeys.push(pubkey.key);
                                            try {
                                                localStorage.setItem(`proc.key_user_id.${remoteKey[index]}.${pubkey.changed}`, pubkey.key);
                                            }
                                            catch (error){
                                                console.warn(error);
                                            }
                                        }
                                    );
                                }
                            };
                            await pubKeyAjax(remoteKeyCsv);
                        }

                        const readableStream = new ReadableStream({
                            start(controller) {
                                controller.enqueue(array);
                                controller.close();
                            }
                        });

                        const recipientsKeys = [];

                        recipientsPubkeys.forEach(
                            async function (entry) {
                                recipientsKeys.push((await openpgp.key.readArmored(entry)).keys[0]);
                            }
                        );

                        await openpgp.key.readArmored(recipientsPubkeys);

                        const options = {
                            message: openpgp.message.fromBinary(readableStream),
                            publicKeys: recipientsKeys,
                            compression: openpgp.enums.compression.zip,
                            privateKeys: privKeyObj,
                        };

                        let startSeconds = new Date().getTime() / 1000;

                        const encrypted = await openpgp.sign(options);
                        const signedArmored = await openpgp.stream.readToEnd(encrypted.data);

                        let authorPubkey = $('input[name=author_pubkey]')[0].value;

                        const optionsVerify = {
                            message: await openpgp.message.readArmored(signedArmored),
                            publicKeys: (await openpgp.key.readArmored(authorPubkey)).keys[0],
                            compression: openpgp.enums.compression.zip,
                        };
                        const verified = await openpgp.verify(optionsVerify);
                        verified.signatures[0].verified.then(
                            (val) => {
                                if (val) {
                                    console.log('signed by key id ' + verified.signatures[0].keyid.toHex());
                                } else {
                                    $('form#-proc-sign-file').prepend(`<div class="messages error">${Drupal.t('Signature could not be verified')}</div>`);
                                    throw new Error('Signature could not be verified');
                                }
                            }
                        );

                        const ciphertext = signedArmored;
                        let endSeconds = new Date().getTime() / 1000,
                            total = endSeconds - startSeconds;

                        $('input[name=cipher_text]')[0].value = ciphertext;
                        $('input[name=source_file_name]')[0].value = files[0].name;
                        $('input[name=source_file_size]')[0].value = files[0].size;
                        $('input[name=source_file_type]')[0].value = files[0].type;
                        $('input[name=source_file_last_change]')[0].value = files[0].lastModified;
                        $('input[name=browser_fingerprint]')[0].value = `${navigator.userAgent}, (${screen.width} x ${screen.height})`;
                        $('input[name=generation_timestamp]')[0].value = startSeconds;
                        $('input[name=generation_timespan]')[0].value = total;
                        // Use keyRingID for allowing a verification fall back to older keyrings.
                        // If the current key is updated, it should still be possible by default
                        // to verify previously signed content:
                        $('input[name=signed]')[0].value = keyRingId;
                        document.getElementById('edit-button').removeAttribute("disabled");
                        document.getElementById('edit-button').value = procJsLabels.proc_save_button_label;
                    }
                };
            }
            if (document.getElementById('edit-upload')) {
                document.getElementById('edit-upload').addEventListener('change', handleFileSelect, false);
            }
        }
    };
})(jQuery);
